 // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //  Author      : Paulo Ricardo ARANTES GILZ / Frédéric CAMPS
 //  Date        : 2018/03
 //  Copyright   : LAAS-CNRS - Toulouse/FRANCE
 // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# include <stdio.h>
# include <stdlib.h>
# include <sys/time.h>
# include <glpk.h>        // LP solver


//////////////////////////////////////////////////////////////////////////
// Global variables
///////////////////////////////////////////////////////////////////////////

struct timeval t_init;
struct timeval t_final;

//////////////////////////////////////////////////////////////////////////
// Headers
///////////////////////////////////////////////////////////////////////////
void    	tic();
void 		toc();

void		solve_problem(char * filename);

//////////////////////////////////////////////////////////////////////////
// Timings
///////////////////////////////////////////////////////////////////////////

void	tic(){

	gettimeofday(&t_init,0);

}

void	toc(){

	gettimeofday(&t_final,0);
	long long elapsed = (t_final.tv_sec-t_init.tv_sec)*1000000LL + t_final.tv_usec-t_init.tv_usec;

	printf("\nElapsed time: %lld microseconds.\n", elapsed);

}


//////////////////////////////////////////////////////////////////////////
// Generates and solves the LP problem
///////////////////////////////////////////////////////////////////////////

void solve_problem(char * filename){

	int j;
	glp_prob *lp;
	lp = glp_create_prob();

	if (glp_read_lp(lp, 0, filename)!=0){		
		exit(-1);
	}

	////////////////////////////////////////////////////////
	//  SOLVING THE PROBLEM
	////////////////////////////////////////////////////////

	tic();	
	//glp_scale_prob(lp,GLP_SF_AUTO); 
	glp_simplex(lp, NULL);
	toc();
	
	double fobj = glp_get_obj_val(lp);
	printf("\nObjective function value: %g\n\n",fobj);

	printf("Impulses:\n"); printf("      ");				
	for(j=1; j<=5; j++)
		printf("N=%8d    ",j);					
	printf("\n");

	printf("DVx  ");				
	for(j=1; j<=5; j++){		
		printf("%11.4e   ",glp_get_col_prim(lp,1+3*(j-1)));			
	}
	printf("\n");

	printf("DVy  ");				
	for(j=1; j<=5; j++){		
		printf("%11.4e   ",glp_get_col_prim(lp,2+3*(j-1)));			
	}
	printf("\n");

	printf("DVz  ");				
	for(j=1; j<=5; j++){		
		printf("%11.4e   ",glp_get_col_prim(lp,3+3*(j-1)));			
	}
	printf("\n");
	
	glp_delete_prob(lp);
	glp_free_env();		
		
}
