// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//  Authors     : Paulo Ricardo ARANTES GILZ / Frédéric CAMPS 
//  Date        : 2018/03
//  Copyright   : LAAS-CNRS - Toulouse/FRANCE
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#include <stdlib.h>
#include <stdio.h>	
#include <sys/time.h>
#include "declarations.h"       // CSDP declarations


struct timeval t_init;
struct timeval t_final;

//////////////////////////////////////////////////////////////////////////
// Headers
///////////////////////////////////////////////////////////////////////////
void    	tic();
void 		toc();
void 		solve_problem(char * prob_name);

//////////////////////////////////////////////////////////////////////////
// Timings
///////////////////////////////////////////////////////////////////////////

void	tic(){

	gettimeofday(&t_init,0);

}

void	toc(){

	gettimeofday(&t_final,0);
	long long elapsed = (t_final.tv_sec-t_init.tv_sec)*1000000LL + t_final.tv_usec-t_init.tv_usec;

	printf("\nElapsed time: %lld microseconds.\n", elapsed);

}

//////////////////////////////////////////////////////////////////////////
// Function solves the problem
///////////////////////////////////////////////////////////////////////////
void solve_problem(char * prob_name){
	
	int N=5, pn, pk, printlevel=2;
	double *pa, *py0, ppobj, pdobj;
	struct blockmatrix pC, pX0, pZ0;
	struct constraintmatrix *pconstraints;


	printf("\nStarting to read problem.\n");
	read_prob(prob_name,&pn,&pk,&pC,&pa,&pconstraints,printlevel);
	printf("Problem read.\n");	

	printf("\nComputing initial solution.\n");
	initsoln(pn,pk,pC,pa,pconstraints,&pX0,&py0,&pZ0);
	printf("Initial solution computed.\n");

	printf("\nSolving problem.\n");
	tic();
	int ret=easy_sdp(pn,pk,pC,pa,pconstraints,0.0,&pX0,&py0,&pZ0,&ppobj,&pdobj);
	toc();
	printf("\nProblem solved.\n");

	printf("\nWriting solution to file.\n");
	write_sol("solution.dat",pn,pk,pX0,py0,pZ0);	
	printf("Solution written to file.\n\n");

	if (ret == 0){
		printf("The objective value is: %15.7f \n\n", -(pdobj+ppobj)/2);
	}
	else{
		printf("SDP failed.\n");	
	}

	printf("Impulses:\n"); printf("      ");				
	for(int i=1; i<=N; i++)
		printf("N=%8d    ",i);				
	printf("\n");

	printf("DVx  ");				
	for(int j=1; j<=N; j++){		
		printf("%11.4e   ",(-pX0.blocks[7].data.vec[1+3*(j-1)]+pX0.blocks[7].data.vec[1+3*N+3*(j-1)]));			
	}
	printf("\n");

	printf("DVy  ");				
	for(int j=1; j<=N; j++){		
		printf("%11.4e   ",(-pX0.blocks[7].data.vec[2+3*(j-1)]+pX0.blocks[7].data.vec[2+3*N+3*(j-1)]));			
	}
	printf("\n");

	printf("DVz  ");				
	for(int j=1; j<=N; j++){		
		printf("%11.4e   ",(-pX0.blocks[7].data.vec[3+3*(j-1)]+pX0.blocks[7].data.vec[3+3*N+3*(j-1)]));			
	}
	printf("\n");

	free_prob(pn,pk,pC,pa,pconstraints,pX0,py0,pZ0);

}

